"""
AsyncAPI documentation generator for Chanx WebSocket consumers.

This package provides utilities to automatically generate AsyncAPI 3.0 specifications
from Chanx WebSocket consumers decorated with @channel, @ws_handler, and @event_handler.
"""
